(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(factory((global.Polestar = {})));
}(this, (function (exports) { 'use strict';

	function createCommonjsModule(fn, module) {
		return module = { exports: {} }, fn(module, module.exports), module.exports;
	}

	var semver = createCommonjsModule(function (module, exports) {
	exports = module.exports = SemVer;

	// The debug function is excluded entirely from the minified version.
	/* nomin */ var debug;
	/* nomin */ if (typeof process === 'object' &&
	    /* nomin */ process.env &&
	    /* nomin */ process.env.NODE_DEBUG &&
	    /* nomin */ /\bsemver\b/i.test(process.env.NODE_DEBUG))
	  /* nomin */ debug = function() {
	    /* nomin */ var args = Array.prototype.slice.call(arguments, 0);
	    /* nomin */ args.unshift('SEMVER');
	    /* nomin */ console.log.apply(console, args);
	    /* nomin */ };
	/* nomin */ else
	  /* nomin */ debug = function() {};

	// Note: this is the semver.org version of the spec that it implements
	// Not necessarily the package version of this code.
	exports.SEMVER_SPEC_VERSION = '2.0.0';

	var MAX_LENGTH = 256;
	var MAX_SAFE_INTEGER = Number.MAX_SAFE_INTEGER || 9007199254740991;

	// Max safe segment length for coercion.
	var MAX_SAFE_COMPONENT_LENGTH = 16;

	// The actual regexps go on exports.re
	var re = exports.re = [];
	var src = exports.src = [];
	var R = 0;

	// The following Regular Expressions can be used for tokenizing,
	// validating, and parsing SemVer version strings.

	// ## Numeric Identifier
	// A single `0`, or a non-zero digit followed by zero or more digits.

	var NUMERICIDENTIFIER = R++;
	src[NUMERICIDENTIFIER] = '0|[1-9]\\d*';
	var NUMERICIDENTIFIERLOOSE = R++;
	src[NUMERICIDENTIFIERLOOSE] = '[0-9]+';


	// ## Non-numeric Identifier
	// Zero or more digits, followed by a letter or hyphen, and then zero or
	// more letters, digits, or hyphens.

	var NONNUMERICIDENTIFIER = R++;
	src[NONNUMERICIDENTIFIER] = '\\d*[a-zA-Z-][a-zA-Z0-9-]*';


	// ## Main Version
	// Three dot-separated numeric identifiers.

	var MAINVERSION = R++;
	src[MAINVERSION] = '(' + src[NUMERICIDENTIFIER] + ')\\.' +
	                   '(' + src[NUMERICIDENTIFIER] + ')\\.' +
	                   '(' + src[NUMERICIDENTIFIER] + ')';

	var MAINVERSIONLOOSE = R++;
	src[MAINVERSIONLOOSE] = '(' + src[NUMERICIDENTIFIERLOOSE] + ')\\.' +
	                        '(' + src[NUMERICIDENTIFIERLOOSE] + ')\\.' +
	                        '(' + src[NUMERICIDENTIFIERLOOSE] + ')';

	// ## Pre-release Version Identifier
	// A numeric identifier, or a non-numeric identifier.

	var PRERELEASEIDENTIFIER = R++;
	src[PRERELEASEIDENTIFIER] = '(?:' + src[NUMERICIDENTIFIER] +
	                            '|' + src[NONNUMERICIDENTIFIER] + ')';

	var PRERELEASEIDENTIFIERLOOSE = R++;
	src[PRERELEASEIDENTIFIERLOOSE] = '(?:' + src[NUMERICIDENTIFIERLOOSE] +
	                                 '|' + src[NONNUMERICIDENTIFIER] + ')';


	// ## Pre-release Version
	// Hyphen, followed by one or more dot-separated pre-release version
	// identifiers.

	var PRERELEASE = R++;
	src[PRERELEASE] = '(?:-(' + src[PRERELEASEIDENTIFIER] +
	                  '(?:\\.' + src[PRERELEASEIDENTIFIER] + ')*))';

	var PRERELEASELOOSE = R++;
	src[PRERELEASELOOSE] = '(?:-?(' + src[PRERELEASEIDENTIFIERLOOSE] +
	                       '(?:\\.' + src[PRERELEASEIDENTIFIERLOOSE] + ')*))';

	// ## Build Metadata Identifier
	// Any combination of digits, letters, or hyphens.

	var BUILDIDENTIFIER = R++;
	src[BUILDIDENTIFIER] = '[0-9A-Za-z-]+';

	// ## Build Metadata
	// Plus sign, followed by one or more period-separated build metadata
	// identifiers.

	var BUILD = R++;
	src[BUILD] = '(?:\\+(' + src[BUILDIDENTIFIER] +
	             '(?:\\.' + src[BUILDIDENTIFIER] + ')*))';


	// ## Full Version String
	// A main version, followed optionally by a pre-release version and
	// build metadata.

	// Note that the only major, minor, patch, and pre-release sections of
	// the version string are capturing groups.  The build metadata is not a
	// capturing group, because it should not ever be used in version
	// comparison.

	var FULL = R++;
	var FULLPLAIN = 'v?' + src[MAINVERSION] +
	                src[PRERELEASE] + '?' +
	                src[BUILD] + '?';

	src[FULL] = '^' + FULLPLAIN + '$';

	// like full, but allows v1.2.3 and =1.2.3, which people do sometimes.
	// also, 1.0.0alpha1 (prerelease without the hyphen) which is pretty
	// common in the npm registry.
	var LOOSEPLAIN = '[v=\\s]*' + src[MAINVERSIONLOOSE] +
	                 src[PRERELEASELOOSE] + '?' +
	                 src[BUILD] + '?';

	var LOOSE = R++;
	src[LOOSE] = '^' + LOOSEPLAIN + '$';

	var GTLT = R++;
	src[GTLT] = '((?:<|>)?=?)';

	// Something like "2.*" or "1.2.x".
	// Note that "x.x" is a valid xRange identifer, meaning "any version"
	// Only the first item is strictly required.
	var XRANGEIDENTIFIERLOOSE = R++;
	src[XRANGEIDENTIFIERLOOSE] = src[NUMERICIDENTIFIERLOOSE] + '|x|X|\\*';
	var XRANGEIDENTIFIER = R++;
	src[XRANGEIDENTIFIER] = src[NUMERICIDENTIFIER] + '|x|X|\\*';

	var XRANGEPLAIN = R++;
	src[XRANGEPLAIN] = '[v=\\s]*(' + src[XRANGEIDENTIFIER] + ')' +
	                   '(?:\\.(' + src[XRANGEIDENTIFIER] + ')' +
	                   '(?:\\.(' + src[XRANGEIDENTIFIER] + ')' +
	                   '(?:' + src[PRERELEASE] + ')?' +
	                   src[BUILD] + '?' +
	                   ')?)?';

	var XRANGEPLAINLOOSE = R++;
	src[XRANGEPLAINLOOSE] = '[v=\\s]*(' + src[XRANGEIDENTIFIERLOOSE] + ')' +
	                        '(?:\\.(' + src[XRANGEIDENTIFIERLOOSE] + ')' +
	                        '(?:\\.(' + src[XRANGEIDENTIFIERLOOSE] + ')' +
	                        '(?:' + src[PRERELEASELOOSE] + ')?' +
	                        src[BUILD] + '?' +
	                        ')?)?';

	var XRANGE = R++;
	src[XRANGE] = '^' + src[GTLT] + '\\s*' + src[XRANGEPLAIN] + '$';
	var XRANGELOOSE = R++;
	src[XRANGELOOSE] = '^' + src[GTLT] + '\\s*' + src[XRANGEPLAINLOOSE] + '$';

	// Coercion.
	// Extract anything that could conceivably be a part of a valid semver
	var COERCE = R++;
	src[COERCE] = '(?:^|[^\\d])' +
	              '(\\d{1,' + MAX_SAFE_COMPONENT_LENGTH + '})' +
	              '(?:\\.(\\d{1,' + MAX_SAFE_COMPONENT_LENGTH + '}))?' +
	              '(?:\\.(\\d{1,' + MAX_SAFE_COMPONENT_LENGTH + '}))?' +
	              '(?:$|[^\\d])';

	// Tilde ranges.
	// Meaning is "reasonably at or greater than"
	var LONETILDE = R++;
	src[LONETILDE] = '(?:~>?)';

	var TILDETRIM = R++;
	src[TILDETRIM] = '(\\s*)' + src[LONETILDE] + '\\s+';
	re[TILDETRIM] = new RegExp(src[TILDETRIM], 'g');
	var tildeTrimReplace = '$1~';

	var TILDE = R++;
	src[TILDE] = '^' + src[LONETILDE] + src[XRANGEPLAIN] + '$';
	var TILDELOOSE = R++;
	src[TILDELOOSE] = '^' + src[LONETILDE] + src[XRANGEPLAINLOOSE] + '$';

	// Caret ranges.
	// Meaning is "at least and backwards compatible with"
	var LONECARET = R++;
	src[LONECARET] = '(?:\\^)';

	var CARETTRIM = R++;
	src[CARETTRIM] = '(\\s*)' + src[LONECARET] + '\\s+';
	re[CARETTRIM] = new RegExp(src[CARETTRIM], 'g');
	var caretTrimReplace = '$1^';

	var CARET = R++;
	src[CARET] = '^' + src[LONECARET] + src[XRANGEPLAIN] + '$';
	var CARETLOOSE = R++;
	src[CARETLOOSE] = '^' + src[LONECARET] + src[XRANGEPLAINLOOSE] + '$';

	// A simple gt/lt/eq thing, or just "" to indicate "any version"
	var COMPARATORLOOSE = R++;
	src[COMPARATORLOOSE] = '^' + src[GTLT] + '\\s*(' + LOOSEPLAIN + ')$|^$';
	var COMPARATOR = R++;
	src[COMPARATOR] = '^' + src[GTLT] + '\\s*(' + FULLPLAIN + ')$|^$';


	// An expression to strip any whitespace between the gtlt and the thing
	// it modifies, so that `> 1.2.3` ==> `>1.2.3`
	var COMPARATORTRIM = R++;
	src[COMPARATORTRIM] = '(\\s*)' + src[GTLT] +
	                      '\\s*(' + LOOSEPLAIN + '|' + src[XRANGEPLAIN] + ')';

	// this one has to use the /g flag
	re[COMPARATORTRIM] = new RegExp(src[COMPARATORTRIM], 'g');
	var comparatorTrimReplace = '$1$2$3';


	// Something like `1.2.3 - 1.2.4`
	// Note that these all use the loose form, because they'll be
	// checked against either the strict or loose comparator form
	// later.
	var HYPHENRANGE = R++;
	src[HYPHENRANGE] = '^\\s*(' + src[XRANGEPLAIN] + ')' +
	                   '\\s+-\\s+' +
	                   '(' + src[XRANGEPLAIN] + ')' +
	                   '\\s*$';

	var HYPHENRANGELOOSE = R++;
	src[HYPHENRANGELOOSE] = '^\\s*(' + src[XRANGEPLAINLOOSE] + ')' +
	                        '\\s+-\\s+' +
	                        '(' + src[XRANGEPLAINLOOSE] + ')' +
	                        '\\s*$';

	// Star ranges basically just allow anything at all.
	var STAR = R++;
	src[STAR] = '(<|>)?=?\\s*\\*';

	// Compile to actual regexp objects.
	// All are flag-free, unless they were created above with a flag.
	for (var i = 0; i < R; i++) {
	  debug(i, src[i]);
	  if (!re[i])
	    re[i] = new RegExp(src[i]);
	}

	exports.parse = parse;
	function parse(version, options) {
	  if (!options || typeof options !== 'object')
	    options = { loose: !!options, includePrerelease: false };

	  if (version instanceof SemVer)
	    return version;

	  if (typeof version !== 'string')
	    return null;

	  if (version.length > MAX_LENGTH)
	    return null;

	  var r = options.loose ? re[LOOSE] : re[FULL];
	  if (!r.test(version))
	    return null;

	  try {
	    return new SemVer(version, options);
	  } catch (er) {
	    return null;
	  }
	}

	exports.valid = valid;
	function valid(version, options) {
	  var v = parse(version, options);
	  return v ? v.version : null;
	}


	exports.clean = clean;
	function clean(version, options) {
	  var s = parse(version.trim().replace(/^[=v]+/, ''), options);
	  return s ? s.version : null;
	}

	exports.SemVer = SemVer;

	function SemVer(version, options) {
	  if (!options || typeof options !== 'object')
	    options = { loose: !!options, includePrerelease: false };
	  if (version instanceof SemVer) {
	    if (version.loose === options.loose)
	      return version;
	    else
	      version = version.version;
	  } else if (typeof version !== 'string') {
	    throw new TypeError('Invalid Version: ' + version);
	  }

	  if (version.length > MAX_LENGTH)
	    throw new TypeError('version is longer than ' + MAX_LENGTH + ' characters')

	  if (!(this instanceof SemVer))
	    return new SemVer(version, options);

	  debug('SemVer', version, options);
	  this.options = options;
	  this.loose = !!options.loose;

	  var m = version.trim().match(options.loose ? re[LOOSE] : re[FULL]);

	  if (!m)
	    throw new TypeError('Invalid Version: ' + version);

	  this.raw = version;

	  // these are actually numbers
	  this.major = +m[1];
	  this.minor = +m[2];
	  this.patch = +m[3];

	  if (this.major > MAX_SAFE_INTEGER || this.major < 0)
	    throw new TypeError('Invalid major version')

	  if (this.minor > MAX_SAFE_INTEGER || this.minor < 0)
	    throw new TypeError('Invalid minor version')

	  if (this.patch > MAX_SAFE_INTEGER || this.patch < 0)
	    throw new TypeError('Invalid patch version')

	  // numberify any prerelease numeric ids
	  if (!m[4])
	    this.prerelease = [];
	  else
	    this.prerelease = m[4].split('.').map(function(id) {
	      if (/^[0-9]+$/.test(id)) {
	        var num = +id;
	        if (num >= 0 && num < MAX_SAFE_INTEGER)
	          return num;
	      }
	      return id;
	    });

	  this.build = m[5] ? m[5].split('.') : [];
	  this.format();
	}

	SemVer.prototype.format = function() {
	  this.version = this.major + '.' + this.minor + '.' + this.patch;
	  if (this.prerelease.length)
	    this.version += '-' + this.prerelease.join('.');
	  return this.version;
	};

	SemVer.prototype.toString = function() {
	  return this.version;
	};

	SemVer.prototype.compare = function(other) {
	  debug('SemVer.compare', this.version, this.options, other);
	  if (!(other instanceof SemVer))
	    other = new SemVer(other, this.options);

	  return this.compareMain(other) || this.comparePre(other);
	};

	SemVer.prototype.compareMain = function(other) {
	  if (!(other instanceof SemVer))
	    other = new SemVer(other, this.options);

	  return compareIdentifiers(this.major, other.major) ||
	         compareIdentifiers(this.minor, other.minor) ||
	         compareIdentifiers(this.patch, other.patch);
	};

	SemVer.prototype.comparePre = function(other) {
	  if (!(other instanceof SemVer))
	    other = new SemVer(other, this.options);

	  // NOT having a prerelease is > having one
	  if (this.prerelease.length && !other.prerelease.length)
	    return -1;
	  else if (!this.prerelease.length && other.prerelease.length)
	    return 1;
	  else if (!this.prerelease.length && !other.prerelease.length)
	    return 0;

	  var i = 0;
	  do {
	    var a = this.prerelease[i];
	    var b = other.prerelease[i];
	    debug('prerelease compare', i, a, b);
	    if (a === undefined && b === undefined)
	      return 0;
	    else if (b === undefined)
	      return 1;
	    else if (a === undefined)
	      return -1;
	    else if (a === b)
	      continue;
	    else
	      return compareIdentifiers(a, b);
	  } while (++i);
	};

	// preminor will bump the version up to the next minor release, and immediately
	// down to pre-release. premajor and prepatch work the same way.
	SemVer.prototype.inc = function(release, identifier) {
	  switch (release) {
	    case 'premajor':
	      this.prerelease.length = 0;
	      this.patch = 0;
	      this.minor = 0;
	      this.major++;
	      this.inc('pre', identifier);
	      break;
	    case 'preminor':
	      this.prerelease.length = 0;
	      this.patch = 0;
	      this.minor++;
	      this.inc('pre', identifier);
	      break;
	    case 'prepatch':
	      // If this is already a prerelease, it will bump to the next version
	      // drop any prereleases that might already exist, since they are not
	      // relevant at this point.
	      this.prerelease.length = 0;
	      this.inc('patch', identifier);
	      this.inc('pre', identifier);
	      break;
	    // If the input is a non-prerelease version, this acts the same as
	    // prepatch.
	    case 'prerelease':
	      if (this.prerelease.length === 0)
	        this.inc('patch', identifier);
	      this.inc('pre', identifier);
	      break;

	    case 'major':
	      // If this is a pre-major version, bump up to the same major version.
	      // Otherwise increment major.
	      // 1.0.0-5 bumps to 1.0.0
	      // 1.1.0 bumps to 2.0.0
	      if (this.minor !== 0 || this.patch !== 0 || this.prerelease.length === 0)
	        this.major++;
	      this.minor = 0;
	      this.patch = 0;
	      this.prerelease = [];
	      break;
	    case 'minor':
	      // If this is a pre-minor version, bump up to the same minor version.
	      // Otherwise increment minor.
	      // 1.2.0-5 bumps to 1.2.0
	      // 1.2.1 bumps to 1.3.0
	      if (this.patch !== 0 || this.prerelease.length === 0)
	        this.minor++;
	      this.patch = 0;
	      this.prerelease = [];
	      break;
	    case 'patch':
	      // If this is not a pre-release version, it will increment the patch.
	      // If it is a pre-release it will bump up to the same patch version.
	      // 1.2.0-5 patches to 1.2.0
	      // 1.2.0 patches to 1.2.1
	      if (this.prerelease.length === 0)
	        this.patch++;
	      this.prerelease = [];
	      break;
	    // This probably shouldn't be used publicly.
	    // 1.0.0 "pre" would become 1.0.0-0 which is the wrong direction.
	    case 'pre':
	      if (this.prerelease.length === 0)
	        this.prerelease = [0];
	      else {
	        var i = this.prerelease.length;
	        while (--i >= 0) {
	          if (typeof this.prerelease[i] === 'number') {
	            this.prerelease[i]++;
	            i = -2;
	          }
	        }
	        if (i === -1) // didn't increment anything
	          this.prerelease.push(0);
	      }
	      if (identifier) {
	        // 1.2.0-beta.1 bumps to 1.2.0-beta.2,
	        // 1.2.0-beta.fooblz or 1.2.0-beta bumps to 1.2.0-beta.0
	        if (this.prerelease[0] === identifier) {
	          if (isNaN(this.prerelease[1]))
	            this.prerelease = [identifier, 0];
	        } else
	          this.prerelease = [identifier, 0];
	      }
	      break;

	    default:
	      throw new Error('invalid increment argument: ' + release);
	  }
	  this.format();
	  this.raw = this.version;
	  return this;
	};

	exports.inc = inc;
	function inc(version, release, loose, identifier) {
	  if (typeof(loose) === 'string') {
	    identifier = loose;
	    loose = undefined;
	  }

	  try {
	    return new SemVer(version, loose).inc(release, identifier).version;
	  } catch (er) {
	    return null;
	  }
	}

	exports.diff = diff;
	function diff(version1, version2) {
	  if (eq(version1, version2)) {
	    return null;
	  } else {
	    var v1 = parse(version1);
	    var v2 = parse(version2);
	    if (v1.prerelease.length || v2.prerelease.length) {
	      for (var key in v1) {
	        if (key === 'major' || key === 'minor' || key === 'patch') {
	          if (v1[key] !== v2[key]) {
	            return 'pre'+key;
	          }
	        }
	      }
	      return 'prerelease';
	    }
	    for (var key in v1) {
	      if (key === 'major' || key === 'minor' || key === 'patch') {
	        if (v1[key] !== v2[key]) {
	          return key;
	        }
	      }
	    }
	  }
	}

	exports.compareIdentifiers = compareIdentifiers;

	var numeric = /^[0-9]+$/;
	function compareIdentifiers(a, b) {
	  var anum = numeric.test(a);
	  var bnum = numeric.test(b);

	  if (anum && bnum) {
	    a = +a;
	    b = +b;
	  }

	  return (anum && !bnum) ? -1 :
	         (bnum && !anum) ? 1 :
	         a < b ? -1 :
	         a > b ? 1 :
	         0;
	}

	exports.rcompareIdentifiers = rcompareIdentifiers;
	function rcompareIdentifiers(a, b) {
	  return compareIdentifiers(b, a);
	}

	exports.major = major;
	function major(a, loose) {
	  return new SemVer(a, loose).major;
	}

	exports.minor = minor;
	function minor(a, loose) {
	  return new SemVer(a, loose).minor;
	}

	exports.patch = patch;
	function patch(a, loose) {
	  return new SemVer(a, loose).patch;
	}

	exports.compare = compare;
	function compare(a, b, loose) {
	  return new SemVer(a, loose).compare(new SemVer(b, loose));
	}

	exports.compareLoose = compareLoose;
	function compareLoose(a, b) {
	  return compare(a, b, true);
	}

	exports.rcompare = rcompare;
	function rcompare(a, b, loose) {
	  return compare(b, a, loose);
	}

	exports.sort = sort;
	function sort(list, loose) {
	  return list.sort(function(a, b) {
	    return exports.compare(a, b, loose);
	  });
	}

	exports.rsort = rsort;
	function rsort(list, loose) {
	  return list.sort(function(a, b) {
	    return exports.rcompare(a, b, loose);
	  });
	}

	exports.gt = gt;
	function gt(a, b, loose) {
	  return compare(a, b, loose) > 0;
	}

	exports.lt = lt;
	function lt(a, b, loose) {
	  return compare(a, b, loose) < 0;
	}

	exports.eq = eq;
	function eq(a, b, loose) {
	  return compare(a, b, loose) === 0;
	}

	exports.neq = neq;
	function neq(a, b, loose) {
	  return compare(a, b, loose) !== 0;
	}

	exports.gte = gte;
	function gte(a, b, loose) {
	  return compare(a, b, loose) >= 0;
	}

	exports.lte = lte;
	function lte(a, b, loose) {
	  return compare(a, b, loose) <= 0;
	}

	exports.cmp = cmp;
	function cmp(a, op, b, loose) {
	  var ret;
	  switch (op) {
	    case '===':
	      if (typeof a === 'object') a = a.version;
	      if (typeof b === 'object') b = b.version;
	      ret = a === b;
	      break;
	    case '!==':
	      if (typeof a === 'object') a = a.version;
	      if (typeof b === 'object') b = b.version;
	      ret = a !== b;
	      break;
	    case '': case '=': case '==': ret = eq(a, b, loose); break;
	    case '!=': ret = neq(a, b, loose); break;
	    case '>': ret = gt(a, b, loose); break;
	    case '>=': ret = gte(a, b, loose); break;
	    case '<': ret = lt(a, b, loose); break;
	    case '<=': ret = lte(a, b, loose); break;
	    default: throw new TypeError('Invalid operator: ' + op);
	  }
	  return ret;
	}

	exports.Comparator = Comparator;
	function Comparator(comp, options) {
	  if (!options || typeof options !== 'object')
	    options = { loose: !!options, includePrerelease: false };

	  if (comp instanceof Comparator) {
	    if (comp.loose === !!options.loose)
	      return comp;
	    else
	      comp = comp.value;
	  }

	  if (!(this instanceof Comparator))
	    return new Comparator(comp, options);

	  debug('comparator', comp, options);
	  this.options = options;
	  this.loose = !!options.loose;
	  this.parse(comp);

	  if (this.semver === ANY)
	    this.value = '';
	  else
	    this.value = this.operator + this.semver.version;

	  debug('comp', this);
	}

	var ANY = {};
	Comparator.prototype.parse = function(comp) {
	  var r = this.options.loose ? re[COMPARATORLOOSE] : re[COMPARATOR];
	  var m = comp.match(r);

	  if (!m)
	    throw new TypeError('Invalid comparator: ' + comp);

	  this.operator = m[1];
	  if (this.operator === '=')
	    this.operator = '';

	  // if it literally is just '>' or '' then allow anything.
	  if (!m[2])
	    this.semver = ANY;
	  else
	    this.semver = new SemVer(m[2], this.options.loose);
	};

	Comparator.prototype.toString = function() {
	  return this.value;
	};

	Comparator.prototype.test = function(version) {
	  debug('Comparator.test', version, this.options.loose);

	  if (this.semver === ANY)
	    return true;

	  if (typeof version === 'string')
	    version = new SemVer(version, this.options);

	  return cmp(version, this.operator, this.semver, this.options);
	};

	Comparator.prototype.intersects = function(comp, options) {
	  if (!(comp instanceof Comparator)) {
	    throw new TypeError('a Comparator is required');
	  }

	  if (!options || typeof options !== 'object')
	    options = { loose: !!options, includePrerelease: false };

	  var rangeTmp;

	  if (this.operator === '') {
	    rangeTmp = new Range(comp.value, options);
	    return satisfies(this.value, rangeTmp, options);
	  } else if (comp.operator === '') {
	    rangeTmp = new Range(this.value, options);
	    return satisfies(comp.semver, rangeTmp, options);
	  }

	  var sameDirectionIncreasing =
	    (this.operator === '>=' || this.operator === '>') &&
	    (comp.operator === '>=' || comp.operator === '>');
	  var sameDirectionDecreasing =
	    (this.operator === '<=' || this.operator === '<') &&
	    (comp.operator === '<=' || comp.operator === '<');
	  var sameSemVer = this.semver.version === comp.semver.version;
	  var differentDirectionsInclusive =
	    (this.operator === '>=' || this.operator === '<=') &&
	    (comp.operator === '>=' || comp.operator === '<=');
	  var oppositeDirectionsLessThan =
	    cmp(this.semver, '<', comp.semver, options) &&
	    ((this.operator === '>=' || this.operator === '>') &&
	    (comp.operator === '<=' || comp.operator === '<'));
	  var oppositeDirectionsGreaterThan =
	    cmp(this.semver, '>', comp.semver, options) &&
	    ((this.operator === '<=' || this.operator === '<') &&
	    (comp.operator === '>=' || comp.operator === '>'));

	  return sameDirectionIncreasing || sameDirectionDecreasing ||
	    (sameSemVer && differentDirectionsInclusive) ||
	    oppositeDirectionsLessThan || oppositeDirectionsGreaterThan;
	};


	exports.Range = Range;
	function Range(range, options) {
	  if (!options || typeof options !== 'object')
	    options = { loose: !!options, includePrerelease: false };

	  if (range instanceof Range) {
	    if (range.loose === !!options.loose &&
	        range.includePrerelease === !!options.includePrerelease) {
	      return range;
	    } else {
	      return new Range(range.raw, options);
	    }
	  }

	  if (range instanceof Comparator) {
	    return new Range(range.value, options);
	  }

	  if (!(this instanceof Range))
	    return new Range(range, options);

	  this.options = options;
	  this.loose = !!options.loose;
	  this.includePrerelease = !!options.includePrerelease;

	  // First, split based on boolean or ||
	  this.raw = range;
	  this.set = range.split(/\s*\|\|\s*/).map(function(range) {
	    return this.parseRange(range.trim());
	  }, this).filter(function(c) {
	    // throw out any that are not relevant for whatever reason
	    return c.length;
	  });

	  if (!this.set.length) {
	    throw new TypeError('Invalid SemVer Range: ' + range);
	  }

	  this.format();
	}

	Range.prototype.format = function() {
	  this.range = this.set.map(function(comps) {
	    return comps.join(' ').trim();
	  }).join('||').trim();
	  return this.range;
	};

	Range.prototype.toString = function() {
	  return this.range;
	};

	Range.prototype.parseRange = function(range) {
	  var loose = this.options.loose;
	  range = range.trim();
	  // `1.2.3 - 1.2.4` => `>=1.2.3 <=1.2.4`
	  var hr = loose ? re[HYPHENRANGELOOSE] : re[HYPHENRANGE];
	  range = range.replace(hr, hyphenReplace);
	  debug('hyphen replace', range);
	  // `> 1.2.3 < 1.2.5` => `>1.2.3 <1.2.5`
	  range = range.replace(re[COMPARATORTRIM], comparatorTrimReplace);
	  debug('comparator trim', range, re[COMPARATORTRIM]);

	  // `~ 1.2.3` => `~1.2.3`
	  range = range.replace(re[TILDETRIM], tildeTrimReplace);

	  // `^ 1.2.3` => `^1.2.3`
	  range = range.replace(re[CARETTRIM], caretTrimReplace);

	  // normalize spaces
	  range = range.split(/\s+/).join(' ');

	  // At this point, the range is completely trimmed and
	  // ready to be split into comparators.

	  var compRe = loose ? re[COMPARATORLOOSE] : re[COMPARATOR];
	  var set = range.split(' ').map(function(comp) {
	    return parseComparator(comp, this.options);
	  }, this).join(' ').split(/\s+/);
	  if (this.options.loose) {
	    // in loose mode, throw out any that are not valid comparators
	    set = set.filter(function(comp) {
	      return !!comp.match(compRe);
	    });
	  }
	  set = set.map(function(comp) {
	    return new Comparator(comp, this.options);
	  }, this);

	  return set;
	};

	Range.prototype.intersects = function(range, options) {
	  if (!(range instanceof Range)) {
	    throw new TypeError('a Range is required');
	  }

	  return this.set.some(function(thisComparators) {
	    return thisComparators.every(function(thisComparator) {
	      return range.set.some(function(rangeComparators) {
	        return rangeComparators.every(function(rangeComparator) {
	          return thisComparator.intersects(rangeComparator, options);
	        });
	      });
	    });
	  });
	};

	// Mostly just for testing and legacy API reasons
	exports.toComparators = toComparators;
	function toComparators(range, options) {
	  return new Range(range, options).set.map(function(comp) {
	    return comp.map(function(c) {
	      return c.value;
	    }).join(' ').trim().split(' ');
	  });
	}

	// comprised of xranges, tildes, stars, and gtlt's at this point.
	// already replaced the hyphen ranges
	// turn into a set of JUST comparators.
	function parseComparator(comp, options) {
	  debug('comp', comp, options);
	  comp = replaceCarets(comp, options);
	  debug('caret', comp);
	  comp = replaceTildes(comp, options);
	  debug('tildes', comp);
	  comp = replaceXRanges(comp, options);
	  debug('xrange', comp);
	  comp = replaceStars(comp, options);
	  debug('stars', comp);
	  return comp;
	}

	function isX(id) {
	  return !id || id.toLowerCase() === 'x' || id === '*';
	}

	// ~, ~> --> * (any, kinda silly)
	// ~2, ~2.x, ~2.x.x, ~>2, ~>2.x ~>2.x.x --> >=2.0.0 <3.0.0
	// ~2.0, ~2.0.x, ~>2.0, ~>2.0.x --> >=2.0.0 <2.1.0
	// ~1.2, ~1.2.x, ~>1.2, ~>1.2.x --> >=1.2.0 <1.3.0
	// ~1.2.3, ~>1.2.3 --> >=1.2.3 <1.3.0
	// ~1.2.0, ~>1.2.0 --> >=1.2.0 <1.3.0
	function replaceTildes(comp, options) {
	  return comp.trim().split(/\s+/).map(function(comp) {
	    return replaceTilde(comp, options);
	  }).join(' ');
	}

	function replaceTilde(comp, options) {
	  if (!options || typeof options !== 'object')
	    options = { loose: !!options, includePrerelease: false };
	  var r = options.loose ? re[TILDELOOSE] : re[TILDE];
	  return comp.replace(r, function(_, M, m, p, pr) {
	    debug('tilde', comp, _, M, m, p, pr);
	    var ret;

	    if (isX(M))
	      ret = '';
	    else if (isX(m))
	      ret = '>=' + M + '.0.0 <' + (+M + 1) + '.0.0';
	    else if (isX(p))
	      // ~1.2 == >=1.2.0 <1.3.0
	      ret = '>=' + M + '.' + m + '.0 <' + M + '.' + (+m + 1) + '.0';
	    else if (pr) {
	      debug('replaceTilde pr', pr);
	      if (pr.charAt(0) !== '-')
	        pr = '-' + pr;
	      ret = '>=' + M + '.' + m + '.' + p + pr +
	            ' <' + M + '.' + (+m + 1) + '.0';
	    } else
	      // ~1.2.3 == >=1.2.3 <1.3.0
	      ret = '>=' + M + '.' + m + '.' + p +
	            ' <' + M + '.' + (+m + 1) + '.0';

	    debug('tilde return', ret);
	    return ret;
	  });
	}

	// ^ --> * (any, kinda silly)
	// ^2, ^2.x, ^2.x.x --> >=2.0.0 <3.0.0
	// ^2.0, ^2.0.x --> >=2.0.0 <3.0.0
	// ^1.2, ^1.2.x --> >=1.2.0 <2.0.0
	// ^1.2.3 --> >=1.2.3 <2.0.0
	// ^1.2.0 --> >=1.2.0 <2.0.0
	function replaceCarets(comp, options) {
	  return comp.trim().split(/\s+/).map(function(comp) {
	    return replaceCaret(comp, options);
	  }).join(' ');
	}

	function replaceCaret(comp, options) {
	  debug('caret', comp, options);
	  if (!options || typeof options !== 'object')
	    options = { loose: !!options, includePrerelease: false };
	  var r = options.loose ? re[CARETLOOSE] : re[CARET];
	  return comp.replace(r, function(_, M, m, p, pr) {
	    debug('caret', comp, _, M, m, p, pr);
	    var ret;

	    if (isX(M))
	      ret = '';
	    else if (isX(m))
	      ret = '>=' + M + '.0.0 <' + (+M + 1) + '.0.0';
	    else if (isX(p)) {
	      if (M === '0')
	        ret = '>=' + M + '.' + m + '.0 <' + M + '.' + (+m + 1) + '.0';
	      else
	        ret = '>=' + M + '.' + m + '.0 <' + (+M + 1) + '.0.0';
	    } else if (pr) {
	      debug('replaceCaret pr', pr);
	      if (pr.charAt(0) !== '-')
	        pr = '-' + pr;
	      if (M === '0') {
	        if (m === '0')
	          ret = '>=' + M + '.' + m + '.' + p + pr +
	                ' <' + M + '.' + m + '.' + (+p + 1);
	        else
	          ret = '>=' + M + '.' + m + '.' + p + pr +
	                ' <' + M + '.' + (+m + 1) + '.0';
	      } else
	        ret = '>=' + M + '.' + m + '.' + p + pr +
	              ' <' + (+M + 1) + '.0.0';
	    } else {
	      debug('no pr');
	      if (M === '0') {
	        if (m === '0')
	          ret = '>=' + M + '.' + m + '.' + p +
	                ' <' + M + '.' + m + '.' + (+p + 1);
	        else
	          ret = '>=' + M + '.' + m + '.' + p +
	                ' <' + M + '.' + (+m + 1) + '.0';
	      } else
	        ret = '>=' + M + '.' + m + '.' + p +
	              ' <' + (+M + 1) + '.0.0';
	    }

	    debug('caret return', ret);
	    return ret;
	  });
	}

	function replaceXRanges(comp, options) {
	  debug('replaceXRanges', comp, options);
	  return comp.split(/\s+/).map(function(comp) {
	    return replaceXRange(comp, options);
	  }).join(' ');
	}

	function replaceXRange(comp, options) {
	  comp = comp.trim();
	  if (!options || typeof options !== 'object')
	    options = { loose: !!options, includePrerelease: false };
	  var r = options.loose ? re[XRANGELOOSE] : re[XRANGE];
	  return comp.replace(r, function(ret, gtlt, M, m, p, pr) {
	    debug('xRange', comp, ret, gtlt, M, m, p, pr);
	    var xM = isX(M);
	    var xm = xM || isX(m);
	    var xp = xm || isX(p);
	    var anyX = xp;

	    if (gtlt === '=' && anyX)
	      gtlt = '';

	    if (xM) {
	      if (gtlt === '>' || gtlt === '<') {
	        // nothing is allowed
	        ret = '<0.0.0';
	      } else {
	        // nothing is forbidden
	        ret = '*';
	      }
	    } else if (gtlt && anyX) {
	      // replace X with 0
	      if (xm)
	        m = 0;
	      if (xp)
	        p = 0;

	      if (gtlt === '>') {
	        // >1 => >=2.0.0
	        // >1.2 => >=1.3.0
	        // >1.2.3 => >= 1.2.4
	        gtlt = '>=';
	        if (xm) {
	          M = +M + 1;
	          m = 0;
	          p = 0;
	        } else if (xp) {
	          m = +m + 1;
	          p = 0;
	        }
	      } else if (gtlt === '<=') {
	        // <=0.7.x is actually <0.8.0, since any 0.7.x should
	        // pass.  Similarly, <=7.x is actually <8.0.0, etc.
	        gtlt = '<';
	        if (xm)
	          M = +M + 1;
	        else
	          m = +m + 1;
	      }

	      ret = gtlt + M + '.' + m + '.' + p;
	    } else if (xm) {
	      ret = '>=' + M + '.0.0 <' + (+M + 1) + '.0.0';
	    } else if (xp) {
	      ret = '>=' + M + '.' + m + '.0 <' + M + '.' + (+m + 1) + '.0';
	    }

	    debug('xRange return', ret);

	    return ret;
	  });
	}

	// Because * is AND-ed with everything else in the comparator,
	// and '' means "any version", just remove the *s entirely.
	function replaceStars(comp, options) {
	  debug('replaceStars', comp, options);
	  // Looseness is ignored here.  star is always as loose as it gets!
	  return comp.trim().replace(re[STAR], '');
	}

	// This function is passed to string.replace(re[HYPHENRANGE])
	// M, m, patch, prerelease, build
	// 1.2 - 3.4.5 => >=1.2.0 <=3.4.5
	// 1.2.3 - 3.4 => >=1.2.0 <3.5.0 Any 3.4.x will do
	// 1.2 - 3.4 => >=1.2.0 <3.5.0
	function hyphenReplace($0,
	                       from, fM, fm, fp, fpr, fb,
	                       to, tM, tm, tp, tpr, tb) {

	  if (isX(fM))
	    from = '';
	  else if (isX(fm))
	    from = '>=' + fM + '.0.0';
	  else if (isX(fp))
	    from = '>=' + fM + '.' + fm + '.0';
	  else
	    from = '>=' + from;

	  if (isX(tM))
	    to = '';
	  else if (isX(tm))
	    to = '<' + (+tM + 1) + '.0.0';
	  else if (isX(tp))
	    to = '<' + tM + '.' + (+tm + 1) + '.0';
	  else if (tpr)
	    to = '<=' + tM + '.' + tm + '.' + tp + '-' + tpr;
	  else
	    to = '<=' + to;

	  return (from + ' ' + to).trim();
	}


	// if ANY of the sets match ALL of its comparators, then pass
	Range.prototype.test = function(version) {
	  if (!version)
	    return false;

	  if (typeof version === 'string')
	    version = new SemVer(version, this.options);

	  for (var i = 0; i < this.set.length; i++) {
	    if (testSet(this.set[i], version, this.options))
	      return true;
	  }
	  return false;
	};

	function testSet(set, version, options) {
	  for (var i = 0; i < set.length; i++) {
	    if (!set[i].test(version))
	      return false;
	  }

	  if (!options)
	    options = {};

	  if (version.prerelease.length && !options.includePrerelease) {
	    // Find the set of versions that are allowed to have prereleases
	    // For example, ^1.2.3-pr.1 desugars to >=1.2.3-pr.1 <2.0.0
	    // That should allow `1.2.3-pr.2` to pass.
	    // However, `1.2.4-alpha.notready` should NOT be allowed,
	    // even though it's within the range set by the comparators.
	    for (var i = 0; i < set.length; i++) {
	      debug(set[i].semver);
	      if (set[i].semver === ANY)
	        continue;

	      if (set[i].semver.prerelease.length > 0) {
	        var allowed = set[i].semver;
	        if (allowed.major === version.major &&
	            allowed.minor === version.minor &&
	            allowed.patch === version.patch)
	          return true;
	      }
	    }

	    // Version has a -pre, but it's not one of the ones we like.
	    return false;
	  }

	  return true;
	}

	exports.satisfies = satisfies;
	function satisfies(version, range, options) {
	  try {
	    range = new Range(range, options);
	  } catch (er) {
	    return false;
	  }
	  return range.test(version);
	}

	exports.maxSatisfying = maxSatisfying;
	function maxSatisfying(versions, range, options) {
	  var max = null;
	  var maxSV = null;
	  try {
	    var rangeObj = new Range(range, options);
	  } catch (er) {
	    return null;
	  }
	  versions.forEach(function (v) {
	    if (rangeObj.test(v)) { // satisfies(v, range, options)
	      if (!max || maxSV.compare(v) === -1) { // compare(max, v, true)
	        max = v;
	        maxSV = new SemVer(max, options);
	      }
	    }
	  });
	  return max;
	}

	exports.minSatisfying = minSatisfying;
	function minSatisfying(versions, range, options) {
	  var min = null;
	  var minSV = null;
	  try {
	    var rangeObj = new Range(range, options);
	  } catch (er) {
	    return null;
	  }
	  versions.forEach(function (v) {
	    if (rangeObj.test(v)) { // satisfies(v, range, options)
	      if (!min || minSV.compare(v) === 1) { // compare(min, v, true)
	        min = v;
	        minSV = new SemVer(min, options);
	      }
	    }
	  });
	  return min;
	}

	exports.validRange = validRange;
	function validRange(range, options) {
	  try {
	    // Return '*' instead of '' so that truthiness works.
	    // This will throw if it's invalid anyway
	    return new Range(range, options).range || '*';
	  } catch (er) {
	    return null;
	  }
	}

	// Determine if version is less than all the versions possible in the range
	exports.ltr = ltr;
	function ltr(version, range, options) {
	  return outside(version, range, '<', options);
	}

	// Determine if version is greater than all the versions possible in the range.
	exports.gtr = gtr;
	function gtr(version, range, options) {
	  return outside(version, range, '>', options);
	}

	exports.outside = outside;
	function outside(version, range, hilo, options) {
	  version = new SemVer(version, options);
	  range = new Range(range, options);

	  var gtfn, ltefn, ltfn, comp, ecomp;
	  switch (hilo) {
	    case '>':
	      gtfn = gt;
	      ltefn = lte;
	      ltfn = lt;
	      comp = '>';
	      ecomp = '>=';
	      break;
	    case '<':
	      gtfn = lt;
	      ltefn = gte;
	      ltfn = gt;
	      comp = '<';
	      ecomp = '<=';
	      break;
	    default:
	      throw new TypeError('Must provide a hilo val of "<" or ">"');
	  }

	  // If it satisifes the range it is not outside
	  if (satisfies(version, range, options)) {
	    return false;
	  }

	  // From now on, variable terms are as if we're in "gtr" mode.
	  // but note that everything is flipped for the "ltr" function.

	  for (var i = 0; i < range.set.length; ++i) {
	    var comparators = range.set[i];

	    var high = null;
	    var low = null;

	    comparators.forEach(function(comparator) {
	      if (comparator.semver === ANY) {
	        comparator = new Comparator('>=0.0.0');
	      }
	      high = high || comparator;
	      low = low || comparator;
	      if (gtfn(comparator.semver, high.semver, options)) {
	        high = comparator;
	      } else if (ltfn(comparator.semver, low.semver, options)) {
	        low = comparator;
	      }
	    });

	    // If the edge version comparator has a operator then our version
	    // isn't outside it
	    if (high.operator === comp || high.operator === ecomp) {
	      return false;
	    }

	    // If the lowest version comparator has an operator and our version
	    // is less than it then it isn't higher than the range
	    if ((!low.operator || low.operator === comp) &&
	        ltefn(version, low.semver)) {
	      return false;
	    } else if (low.operator === ecomp && ltfn(version, low.semver)) {
	      return false;
	    }
	  }
	  return true;
	}

	exports.prerelease = prerelease;
	function prerelease(version, options) {
	  var parsed = parse(version, options);
	  return (parsed && parsed.prerelease.length) ? parsed.prerelease : null;
	}

	exports.intersects = intersects;
	function intersects(r1, r2, options) {
	  r1 = new Range(r1, options);
	  r2 = new Range(r2, options);
	  return r1.intersects(r2)
	}

	exports.coerce = coerce;
	function coerce(version) {
	  if (version instanceof SemVer)
	    return version;

	  if (typeof version !== 'string')
	    return null;

	  var match = version.match(re[COERCE]);

	  if (match == null)
	    return null;

	  return parse((match[1] || '0') + '.' + (match[2] || '0') + '.' + (match[3] || '0')); 
	}
	});
	var semver_1 = semver.SEMVER_SPEC_VERSION;
	var semver_2 = semver.re;
	var semver_3 = semver.src;
	var semver_4 = semver.parse;
	var semver_5 = semver.valid;
	var semver_6 = semver.clean;
	var semver_7 = semver.SemVer;
	var semver_8 = semver.inc;
	var semver_9 = semver.diff;
	var semver_10 = semver.compareIdentifiers;
	var semver_11 = semver.rcompareIdentifiers;
	var semver_12 = semver.major;
	var semver_13 = semver.minor;
	var semver_14 = semver.patch;
	var semver_15 = semver.compare;
	var semver_16 = semver.compareLoose;
	var semver_17 = semver.rcompare;
	var semver_18 = semver.sort;
	var semver_19 = semver.rsort;
	var semver_20 = semver.gt;
	var semver_21 = semver.lt;
	var semver_22 = semver.eq;
	var semver_23 = semver.neq;
	var semver_24 = semver.gte;
	var semver_25 = semver.lte;
	var semver_26 = semver.cmp;
	var semver_27 = semver.Comparator;
	var semver_28 = semver.Range;
	var semver_29 = semver.toComparators;
	var semver_30 = semver.satisfies;
	var semver_31 = semver.maxSatisfying;
	var semver_32 = semver.minSatisfying;
	var semver_33 = semver.validRange;
	var semver_34 = semver.ltr;
	var semver_35 = semver.gtr;
	var semver_36 = semver.outside;
	var semver_37 = semver.prerelease;
	var semver_38 = semver.intersects;
	var semver_39 = semver.coerce;

	(function (ResolutionType) {
	    ResolutionType["Available"] = "available";
	    ResolutionType["LoadFrom"] = "load-from";
	})(exports.ResolutionType || (exports.ResolutionType = {}));

	var NPMURLPattern = /^(?:npm:\/\/)?((?:@[\w\.\-]+\/)?\w[\w\.\-]+)(@[^\/]+)?(\/.*)?$/;
	function splitNPMURL(url) {
	    var loaders = extractLoaders(url);
	    var requestWithoutLoaders = removeLoaders(url);
	    var match = requestWithoutLoaders.match(NPMURLPattern);
	    if (match) {
	        var _ = match[0], name_1 = match[1], version = match[2], pathname = match[3];
	        return {
	            loaders: loaders,
	            name: name_1,
	            version: version,
	            pathname: pathname,
	        };
	    }
	}
	var DefaultResolver = /** @class */ (function () {
	    function DefaultResolver() {
	        // Maps known URLs to their module id.
	        this.knownURLs = {};
	        // Records redirects due to extensions, checking index.js, the package.json's
	        // `main` field, etc.
	        this.npmPathRedirects = {};
	        // For each package, store a list of versions that we have any ids for.
	        this.npmKnownPackageVersions = {};
	        // Keep track of version aliases of each package, so that if an alias is
	        // referenced in multiple places, it can be immediately resolved to the
	        // appropriate actual version.
	        this.npmVersionAliases = {};
	        // Keep track of the first used version for each package, and use that for all
	        // subsequent bare references to the same package.
	        this.npmDefaultVersions = {};
	    }
	    DefaultResolver.prototype.registerId = function (id) {
	        this.knownURLs[id] = id;
	    };
	    // Register an id along with the URL that requested it, to speed up future
	    // resolutions.
	    DefaultResolver.prototype.registerResolvedURL = function (url, id) {
	        this.knownURLs[url] = id;
	        var idNPMMatch = removeLoaders(id).match(NPMURLPattern);
	        if (idNPMMatch) {
	            var _ = idNPMMatch[0], name_2 = idNPMMatch[1], version = idNPMMatch[2], _a = idNPMMatch[3], path = _a === void 0 ? '' : _a;
	            var urlNPMMatch = removeLoaders(url).match(NPMURLPattern);
	            if (urlNPMMatch) {
	                var urlNPMVersion = urlNPMMatch[2] || '';
	                if (urlNPMVersion !== version) {
	                    // We've received an actual version in response to an alias or range.
	                    this.npmVersionAliases[urlNPMVersion] = version;
	                }
	                var urlNPMPath = urlNPMMatch[3] || '';
	                if (urlNPMPath !== path) {
	                    this.npmPathRedirects[name_2 + version + urlNPMPath] = path;
	                }
	            }
	            // For each package, store a list of versions that we have any ids for,
	            // so that we can match ranges against known versions.
	            if (!this.npmKnownPackageVersions[name_2]) {
	                this.npmKnownPackageVersions[name_2] = [version];
	            }
	            else {
	                this.npmKnownPackageVersions[name_2].push(version);
	            }
	        }
	    };
	    // Either returns an { id } object containing a registered module id, or a
	    // { url } object containing a URL that can be resolved to a module id/source
	    // by the breadboard host.
	    //
	    // ids are all urls, but urls aren't necessarily ids.
	    DefaultResolver.prototype.resolve = function (request, parentId, packageDependencies) {
	        var url;
	        var loaders = extractLoaders(request);
	        var requestWithoutLoaders = removeLoaders(request);
	        var npmMatch = requestWithoutLoaders.match(NPMURLPattern);
	        if (!npmMatch) {
	            // It's not bare, so resolve the URL relative to the parent
	            // and then try to match it against our npm pattern again.
	            url = resolveIfNotPlainOrUrl(requestWithoutLoaders, parentId);
	            npmMatch = url.match(NPMURLPattern);
	        }
	        var name, version, path;
	        if (npmMatch) {
	            name = npmMatch[1];
	            version = npmMatch[2];
	            path = npmMatch[3] || '';
	            // If we got no version or a version alias, try and match it to a
	            // real version.
	            if (!version) {
	                var packageVersion = packageDependencies[name];
	                if (packageVersion) {
	                    packageVersion = '@' + packageDependencies[name];
	                }
	                version = packageVersion || this.npmDefaultVersions[name] || '@latest';
	            }
	            if (this.npmVersionAliases[name + version]) {
	                version = this.npmVersionAliases[name + version];
	            }
	            else if (!semver_5(version.slice(1))) {
	                version = this.getKnownMatchingVersion(name, version) || version;
	            }
	            // If this is the first time we've seen this package, store the version
	            // we're using for future versionless references.
	            if (!this.npmDefaultVersions[name]) {
	                this.npmDefaultVersions[name] = version;
	            }
	            path = this.npmPathRedirects[name + version + path] || path;
	            url = 'npm://' + name + version + path;
	        }
	        // If we've seen this URL before, immediately return the corresponding id.
	        var id = this.knownURLs[url];
	        if (id) {
	            return { type: exports.ResolutionType.Available, id: loaders + id };
	        }
	        return { type: exports.ResolutionType.LoadFrom, url: loaders + url };
	    };
	    DefaultResolver.prototype.getKnownMatchingVersion = function (name, range) {
	        var rangeWithoutSymbol = range.slice(1);
	        if (semver_33(rangeWithoutSymbol)) {
	            var versions = this.npmKnownPackageVersions[name] || [];
	            for (var i = 0; i < versions.length; i++) {
	                var version = versions[i];
	                if (semver_30(version.slice(1), rangeWithoutSymbol)) {
	                    return version;
	                }
	            }
	        }
	    };
	    return DefaultResolver;
	}());
	/**
	 * From system.js
	 * Copyright (C) 2013-2018 Guy Bedford
	 * https://github.com/systemjs/systemjs
	 */
	var backslashRegEx = /\\/g;
	function resolveIfNotPlainOrUrl(relUrl, parentUrl) {
	    if (relUrl.indexOf('://') >= 0) {
	        return relUrl;
	    }
	    if (relUrl.indexOf('\\') !== -1)
	        relUrl = relUrl.replace(backslashRegEx, '/');
	    // protocol-relative
	    if (relUrl[0] === '/' && relUrl[1] === '/') {
	        return parentUrl.slice(0, parentUrl.indexOf(':') + 1) + relUrl;
	    }
	    // relative-url
	    else if (relUrl[0] === '.' && (relUrl[1] === '/' || relUrl[1] === '.' && (relUrl[2] === '/' || relUrl.length === 2 && (relUrl += '/')) ||
	        relUrl.length === 1 && (relUrl += '/')) ||
	        relUrl[0] === '/') {
	        var parentProtocol = parentUrl.slice(0, parentUrl.indexOf(':') + 1);
	        // Disabled, but these cases will give inconsistent results for deep backtracking
	        //if (parentUrl[parentProtocol.length] !== '/')
	        //  throw new Error('Cannot resolve');
	        // read pathname from parent URL
	        // pathname taken to be part after leading "/"
	        var pathname = void 0;
	        if (parentUrl[parentProtocol.length + 1] === '/') {
	            // resolving to a :// so we need to read out the auth and host
	            if (parentProtocol !== 'file:') {
	                pathname = parentUrl.slice(parentProtocol.length + 2);
	                pathname = pathname.slice(pathname.indexOf('/') + 1);
	            }
	            else {
	                pathname = parentUrl.slice(8);
	            }
	        }
	        else {
	            // resolving to :/ so pathname is the /... part
	            pathname = parentUrl.slice(parentProtocol.length + (parentUrl[parentProtocol.length] === '/'));
	        }
	        if (relUrl[0] === '/')
	            return parentUrl.slice(0, parentUrl.length - pathname.length - 1) + relUrl;
	        // join together and split for removal of .. and . segments
	        // looping the string instead of anything fancy for perf reasons
	        // '../../../../../z' resolved to 'x/y' is just 'z'
	        var segmented = pathname.slice(0, pathname.lastIndexOf('/') + 1) + relUrl;
	        var output = [];
	        var segmentIndex = -1;
	        for (var i = 0; i < segmented.length; i++) {
	            // busy reading a segment - only terminate on '/'
	            if (segmentIndex !== -1) {
	                if (segmented[i] === '/') {
	                    output.push(segmented.slice(segmentIndex, i + 1));
	                    segmentIndex = -1;
	                }
	            }
	            // new segment - check if it is relative
	            else if (segmented[i] === '.') {
	                // ../ segment
	                if (segmented[i + 1] === '.' && (segmented[i + 2] === '/' || i + 2 === segmented.length)) {
	                    output.pop();
	                    i += 2;
	                }
	                // ./ segment
	                else if (segmented[i + 1] === '/' || i + 1 === segmented.length) {
	                    i += 1;
	                }
	                else {
	                    // the start of a new segment as below
	                    segmentIndex = i;
	                }
	            }
	            // it is the start of a new segment
	            else {
	                segmentIndex = i;
	            }
	        }
	        // finish reading out the last segment
	        if (segmentIndex !== -1)
	            output.push(segmented.slice(segmentIndex));
	        return parentUrl.slice(0, parentUrl.length - pathname.length) + output.join('');
	    }
	}
	function extractLoaders(url) {
	    var match = url.match(/^(.*!)*/);
	    return match ? match[0] : '';
	}
	function removeLoaders(url) {
	    return url.replace(/^(.*!)*/, '');
	}

	/*! *****************************************************************************
	Copyright (c) Microsoft Corporation. All rights reserved.
	Licensed under the Apache License, Version 2.0 (the "License"); you may not use
	this file except in compliance with the License. You may obtain a copy of the
	License at http://www.apache.org/licenses/LICENSE-2.0

	THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
	KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
	WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
	MERCHANTABLITY OR NON-INFRINGEMENT.

	See the Apache Version 2.0 License for specific language governing permissions
	and limitations under the License.
	***************************************************************************** */
	/* global Reflect, Promise */

	var extendStatics = function(d, b) {
	    extendStatics = Object.setPrototypeOf ||
	        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
	        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
	    return extendStatics(d, b);
	};

	function __extends(d, b) {
	    extendStatics(d, b);
	    function __() { this.constructor = d; }
	    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
	}

	var PolestarError = /** @class */ (function (_super) {
	    __extends(PolestarError, _super);
	    function PolestarError() {
	        var _newTarget = this.constructor;
	        var args = [];
	        for (var _i = 0; _i < arguments.length; _i++) {
	            args[_i] = arguments[_i];
	        }
	        var _this = _super.apply(this, args) || this;
	        Object.setPrototypeOf(_this, _newTarget.prototype);
	        _this.name = _newTarget.name;
	        // Maintains proper stack trace for where our error was thrown (only available on V8)
	        if (Error.captureStackTrace) {
	            Error.captureStackTrace(_this, _newTarget);
	        }
	        return _this;
	    }
	    return PolestarError;
	}(Error));
	var UnresolvableError = /** @class */ (function (_super) {
	    __extends(UnresolvableError, _super);
	    function UnresolvableError(request, fromModuleId) {
	        var _newTarget = this.constructor;
	        var _this = _super.call(this, "Module \"" + request + "\" could not be resolved by require.resolve from module \"" + fromModuleId + "\".") || this;
	        Object.setPrototypeOf(_this, _newTarget.prototype);
	        _this.request = request;
	        _this.fromModuleId = fromModuleId;
	        return _this;
	    }
	    return UnresolvableError;
	}(PolestarError));
	var CyclicDependencyError = /** @class */ (function (_super) {
	    __extends(CyclicDependencyError, _super);
	    function CyclicDependencyError(request) {
	        var _this = _super.call(this, "Module \"" + request + "\" can not be required from itself.") || this;
	        _this.request = request;
	        return _this;
	    }
	    return CyclicDependencyError;
	}(PolestarError));

	var nextScriptId = 1;
	var scriptIdPrefix = Math.random().toString(36).slice(2);
	var __polestar_script__ = function (id, fn) {
	    __polestar_script__[id] = fn;
	};
	if (!window['__polestar_script__']) {
	    window['__polestar_script__'] = __polestar_script__;
	}
	else {
	    throw new Error("Cannot load multiple copies of Polestar.");
	}
	// Running code with `eval` will cause some errors messages to be hidden from
	// the script due to Chrome treating them as originating from another origin.
	// Evaluating code by running it in script tags is supposedly slower, but
	// allows errors to be properly forwarded to non-native consoles.
	function createFunction(argNames, source) {
	    var scriptId = scriptIdPrefix + '/' + nextScriptId++;
	    var script = document.createElement('script');
	    script.innerHTML =
	        "(function(){__polestar_script__(" + JSON.stringify(scriptId) + ", function module(" + argNames.join(',') + ") {" +
	            source +
	            "\n})})()";
	    // Script tags with inline source are synchronously executed when appended
	    // to the document body.
	    document.body.appendChild(script);
	    var result = __polestar_script__[scriptId];
	    delete __polestar_script__[scriptId];
	    document.body.removeChild(script);
	    return result;
	}

	function createPolestar(options) {
	    if (options === void 0) { options = {}; }
	    return new Polestar(options);
	}
	var Polestar = /** @class */ (function () {
	    function Polestar(options) {
	        var _this = this;
	        this.hasCalledOnEntry = false;
	        this.nextEntryId = 1;
	        this.handleFetchResult = function (result) {
	            if (_this.error) {
	                return Promise.reject(_this.error);
	            }
	            var url = result.url;
	            var id = result.id;
	            var load = _this.loads[url];
	            _this.resolver.registerResolvedURL(url, id);
	            // It's possible that this id has already been loaded using a different url.
	            // If so, we need to let it know of any more requesting modules, in case
	            // they're needed to avoid deadlock.
	            var moduleWrapper = _this.moduleWrappers[id];
	            if (moduleWrapper) {
	                moduleWrapper.addToRequiredBy(load.requiredBy);
	                return Promise.resolve(moduleWrapper);
	            }
	            return _this.prepareModuleWrapper(id, result.code, result.dependencies || [], result.dependencyVersionRanges || {}, load.requiredBy, result.css);
	        };
	        this.handleUnrequiredPrepared = function (moduleWrapper) {
	            if (!_this.hasCalledOnEntry) {
	                _this.options.onEntry();
	            }
	            moduleWrapper.execute();
	            return moduleWrapper;
	        };
	        // If we encounter any error while loading/executing modules, stop loading
	        // subsequent modules and notify our owner.
	        this.setError = function (error) {
	            var lastError = _this.error;
	            _this.error = error;
	            if (lastError !== error) {
	                if (_this.options.onError) {
	                    _this.options.onError(error);
	                }
	                else {
	                    console.error("Error encountered while loading module:", error);
	                }
	            }
	        };
	        this.options = Object.assign({ globals: {}, onEntry: function () { } }, options);
	        this.loads = {};
	        this.moduleWrappers = {};
	        this.resolver = options.resolver || new DefaultResolver();
	    }
	    Polestar.prototype.evaluate = function (dependencies, code, dependencyVersionRanges, id) {
	        if (id === undefined) {
	            id = 'polestar-anonymous://' + String(this.nextEntryId++);
	        }
	        return this.prepareModuleWrapper(id, code, dependencies || [], dependencyVersionRanges || {}).then(getModule);
	    };
	    Polestar.prototype.require = function (request) {
	        var resolution = this.resolve(request);
	        var url = resolution.type === exports.ResolutionType.Available ? resolution.id : resolution.url;
	        var modulePromise = this.loadWrapper(url).then(getModule);
	        modulePromise.catch(this.setError);
	        return modulePromise;
	    };
	    Polestar.prototype.resolve = function (request, parentId, defaultPackageVersionRanges) {
	        if (defaultPackageVersionRanges === void 0) { defaultPackageVersionRanges = {}; }
	        return this.resolver.resolve(request, parentId, defaultPackageVersionRanges);
	    };
	    // Initializes a module if it doesn't already exist
	    Polestar.prototype.loadWrapper = function (url, requiredBy, originalRequest) {
	        var moduleWrapper = this.moduleWrappers[url];
	        if (moduleWrapper) {
	            return Promise.resolve(moduleWrapper);
	        }
	        var existingLoad = this.loads[url];
	        if (existingLoad) {
	            if (requiredBy) {
	                existingLoad.requiredBy.push(requiredBy);
	            }
	            return existingLoad.preparedModuleWrapperPromise;
	        }
	        var preparedModuleWrapperPromise = this.options.fetcher(url, {
	            requiredById: requiredBy && requiredBy.module.id,
	            originalRequest: originalRequest,
	        })
	            .then(this.handleFetchResult);
	        this.loads[url] = {
	            requiredBy: requiredBy ? [requiredBy] : [],
	            preparedModuleWrapperPromise: preparedModuleWrapperPromise,
	        };
	        return preparedModuleWrapperPromise;
	    };
	    Polestar.prototype.prepareModuleWrapper = function (id, code, dependencies, dependencyVersionRanges, requiredBy, css) {
	        var _this = this;
	        if (requiredBy === void 0) { requiredBy = []; }
	        if (this.error) {
	            return Promise.reject(this.error);
	        }
	        if (css) {
	            var head = document.head || document.getElementsByTagName('head')[0];
	            var style = document.createElement('style');
	            style.type = 'text/css';
	            style.appendChild(document.createTextNode(css));
	            head.appendChild(style);
	        }
	        try {
	            var moduleWrapper_1;
	            var globals = this.options.globals || {};
	            var globalNames = Object.keys(globals);
	            var globalValues = [];
	            for (var i = 0; i < globalNames.length; i++) {
	                globalValues.push(globals[globalNames[i]]);
	            }
	            var prepareDependencies_1;
	            // For UMD modules, we'll use their dependencies instead of data.prepareRequires.
	            if (dependencies === 'umd') {
	                var define = function (name, dependencies, factory) {
	                    // name and dependencies are optional
	                    if (!factory) {
	                        factory = dependencies;
	                        dependencies = Array.isArray(name) ? name : [];
	                        if (!factory) {
	                            factory = name;
	                            dependencies = [];
	                        }
	                    }
	                    // Some UMD packages depend on an "exports" object, which shouldn't
	                    // be required like normal objects.
	                    prepareDependencies_1 = dependencies.slice(0);
	                    var index = prepareDependencies_1.indexOf('exports');
	                    if (index !== -1) {
	                        prepareDependencies_1.splice(index, 1);
	                    }
	                    moduleWrapper_1 = new ModuleWrapper(_this, id, dependencyVersionRanges, function (require, module, exports) {
	                        var factoryResult = factory.apply(null, dependencies.map(function (dep) { return dep === 'exports' ? exports : require(dep); }));
	                        if (factoryResult !== undefined) {
	                            module.exports = factoryResult;
	                        }
	                    });
	                };
	                define.amd = true;
	                var fn = createFunction(['define'].concat(globalNames), code);
	                var args = [define].concat(globalValues);
	                fn.apply(this.options.moduleThis, args);
	            }
	            else {
	                globalNames.push('require', 'module', 'exports');
	                prepareDependencies_1 = dependencies;
	                var moduleFunction = createFunction(globalNames, code);
	                var boundModuleFunction = Function.prototype.bind.apply(moduleFunction, [this.options.moduleThis].concat(globalValues));
	                moduleWrapper_1 = new ModuleWrapper(this, id, dependencyVersionRanges, boundModuleFunction);
	            }
	            this.resolver.registerId(id);
	            this.moduleWrappers[id] = moduleWrapper_1;
	            var preparedPromise = moduleWrapper_1.prepare(prepareDependencies_1, requiredBy);
	            if (requiredBy.length === 0) {
	                preparedPromise.catch(this.setError);
	                preparedPromise = preparedPromise.then(this.handleUnrequiredPrepared);
	                preparedPromise.catch(this.setError);
	            }
	            return preparedPromise;
	        }
	        catch (error) {
	            this.setError(error);
	            return Promise.reject(error);
	        }
	    };
	    return Polestar;
	}());
	var ModuleWrapper = /** @class */ (function () {
	    function ModuleWrapper(loader, id, dependencyVersionRanges, fn) {
	        var _this = this;
	        this.stopWaitingFor = function (url) {
	            var index = _this.waitingFor.indexOf(url);
	            if (index !== -1) {
	                _this.waitingFor.splice(index, 1);
	                if (_this.waitingFor.length === 0) {
	                    _this.isPrepared = true;
	                    _this.resolvePrepared(_this);
	                }
	            }
	        };
	        this.addRequiredBy = function (moduleWrapper) {
	            _this.requiredBy.add(moduleWrapper);
	        };
	        this.requiredBy = new Set();
	        this.waitingFor = [];
	        this.fn = fn;
	        this.polestar = loader;
	        this.dependencyVersionRanges = dependencyVersionRanges;
	        this.isPrepared;
	        this.preparedPromise = new Promise(function (resolve, reject) {
	            _this.resolvePrepared = resolve;
	            _this.rejectPrepared = reject;
	        });
	        var require = (function (request) {
	            var requestedId = require.resolve(request);
	            if (id === requestedId) {
	                throw new CyclicDependencyError(id);
	            }
	            var requestedWrapper = loader.moduleWrappers[requestedId];
	            if (!requestedWrapper.module.loaded) {
	                requestedWrapper.execute();
	            }
	            return requestedWrapper.module.exports;
	        });
	        require.resolve = function (request) {
	            var result = _this.polestar.resolve(request, id, dependencyVersionRanges);
	            if (result.type !== exports.ResolutionType.Available) {
	                throw new UnresolvableError(request, id);
	            }
	            return result.id;
	        };
	        this.module = {
	            exports: {},
	            id: id,
	            loaded: false,
	            require: require,
	        };
	    }
	    // Actually execute the module code
	    ModuleWrapper.prototype.execute = function () {
	        var module = this.module;
	        // This should never happen, but just in case...
	        if (module.loaded) {
	            throw new Error("Can't execute module " + this.module.id + " twice.");
	        }
	        module.loaded = true;
	        this.fn(module.require, module, module.exports);
	    };
	    // This is separate from the constructor, as it allows the loader to
	    // register this module under multiple URLs before loading/preparing any
	    // of the module's dependencies.
	    ModuleWrapper.prototype.prepare = function (dependencyRequests, requiredByWrappers) {
	        var _this = this;
	        if (dependencyRequests === void 0) { dependencyRequests = []; }
	        if (requiredByWrappers === void 0) { requiredByWrappers = []; }
	        // Build a set of all all modules depending on us, modules depending on
	        // modules depending on us, etc.
	        for (var i = 0; i < requiredByWrappers.length; i++) {
	            var requiredBy = requiredByWrappers[i];
	            this.addRequiredBy(requiredBy);
	            requiredBy.requiredBy.forEach(this.addRequiredBy);
	        }
	        var _loop_1 = function (i) {
	            var request = dependencyRequests[i];
	            if (request) {
	                var result = this_1.polestar.resolve(request, this_1.module.id, this_1.dependencyVersionRanges);
	                if (result.type === exports.ResolutionType.Available) {
	                    var dependencyWrapper = this_1.polestar.moduleWrappers[result.id];
	                    // If the requested module depends on us, then waiting for the
	                    // requested module to be ready will result in deadlock.
	                    if (!this_1.requiredBy.has(dependencyWrapper) && !dependencyWrapper.isPrepared) {
	                        this_1.waitingFor.push(result.id);
	                        dependencyWrapper.preparedPromise
	                            .then(getWrapperId)
	                            .then(this_1.stopWaitingFor)
	                            .catch(this_1.rejectPrepared);
	                    }
	                }
	                else {
	                    var url_1 = result.url;
	                    this_1.waitingFor.push(result.url);
	                    this_1.polestar.loadWrapper(url_1, this_1, request)
	                        .then(function () { return _this.stopWaitingFor(url_1); })
	                        .catch(this_1.rejectPrepared);
	                }
	            }
	        };
	        var this_1 = this;
	        for (var i = 0; i < dependencyRequests.length; i++) {
	            _loop_1(i);
	        }
	        if (!this.waitingFor.length) {
	            this.isPrepared = true;
	            this.resolvePrepared(this);
	        }
	        return this.preparedPromise;
	    };
	    ModuleWrapper.prototype.addToRequiredBy = function (requiredByWrappers) {
	        // Update our list of modules that are waiting for us, any modules
	        // that are waiting for modules that are waiting for us, etc.
	        for (var i = 0; i < requiredByWrappers.length; i++) {
	            var requiredBy = requiredByWrappers[i];
	            this.addRequiredBy(requiredBy);
	            requiredBy.requiredBy.forEach(this.addRequiredBy);
	        }
	        // Stop waiting for any modules that are waiting for us, any modules
	        // that are waiting for modules that are waiting for us, etc.
	        for (var i = 0; i < this.waitingFor.length; i++) {
	            var waitingForURL = this.waitingFor[i];
	            var result = this.polestar.resolve(waitingForURL, this.module.id, this.dependencyVersionRanges);
	            // It can't be waiting for us if it hasn't loaded yet
	            if (result.type === exports.ResolutionType.Available && this.requiredBy.has(this.polestar.moduleWrappers[result.id])) {
	                this.stopWaitingFor(waitingForURL);
	            }
	        }
	    };
	    return ModuleWrapper;
	}());
	function getWrapperId(moduleWrapper) {
	    return moduleWrapper.module.id;
	}
	function getModule(moduleWrapper) {
	    return moduleWrapper.module;
	}

	exports.splitNPMURL = splitNPMURL;
	exports.DefaultResolver = DefaultResolver;
	exports.PolestarError = PolestarError;
	exports.UnresolvableError = UnresolvableError;
	exports.CyclicDependencyError = CyclicDependencyError;
	exports.createPolestar = createPolestar;
	exports.Polestar = Polestar;
	exports.ModuleWrapper = ModuleWrapper;

	Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=polestar.js.map
